/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"

/* a wrapper function for the yes/no/ok popup curses windows	*/
int do_popup(char *msg, int type) {
	int retval = 0;

	/* check what kind of popup was called and do the popup	*/
	switch(type) {
		case YN:
			/* yes/no popup, YES as default			*/
			retval = popup_yesno(msg, TRUE);
			break;
		case YNN:
			/* yes/no popup, NO as default			*/
			retval = popup_yesno(msg, FALSE);
			break;
		case HIT_OK:
			/* plain ole "PRESS OK" popup			*/
			popup_ok(msg, COLOR_PAIR(COLOR_CYAN)|A_BOLD);
			retval = -1;
			break;
		case PERR:
			popup_ok(msg, COLOR_PAIR(COLOR_RED)|A_BOLD);
	}

	return retval;	/* return value of user input (if any)	*/
}

/* function to get a value from the user - I use like strcpy(target, popup_getval()); */
char *popup_getval(char *msg) {
        int x, y, w, h;	/* a character, x/y cordinates, and width/height definitions 	*/
	WINDOW *win;
	
	x = ((LINES / 2) - 5);				/* center X cordinate	*/
	y = 0;	/* center Y cordinate	*/
	w = COLS;				/* figure width		*/
	h = 5;						/* all these are 5 high	*/

	win = newwin(h, w, x, y);			/* create a new window	*/
	
        keypad(win, TRUE);				/* enable use of keypad	*/

        wmove(win, 1, 1);

        if(color) wattron(win, COLOR_PAIR(COLOR_CYAN));
        wprintw(win, "%s\n", msg);
        if(color) wstandend(win);

        wmove(win, 3, 1);
        wprintw(win, ": ");
        box(win, ACS_VLINE, ACS_HLINE);
        
	wrefresh(win);
        echo();
        wgetnstr(win, data, (w - 5));
        noecho();
	delwin(win);
	return (data);

}

/* When doing code cleanup, merge the below functions; they share a lot of the same code, and
 should really just "print OK" or "print YES/NO default yes", or "print YES/NO default no".
 without having all this duplicated code. */

int popup_ok(char *msg, int color_pair) {
        int ch, x, y, w, h;	/* a character, x/y cordinates, and width/height definitions 	*/
	WINDOW *win;
	
	x = ((LINES / 2) - 5);				/* center X cordinate	*/
	y = ((COLS / 2) - (strlen(msg) / 2) - 4);	/* center Y cordinate	*/
	w = (strlen(msg) + 4);				/* figure width		*/
	h = 5;						/* all these are 5 high	*/

	if(w >= COLS || (w + y) >= COLS) {
	       	w = COLS;
		y = 0;
	}
	
	win = newwin(h, w, x, y);			/* create a new window	*/
	
        keypad(win, TRUE);				/* enable use of keypad	*/
        ch = 'Z';					/* set char to something benign	*/

        while(ch != 13) {
                noecho();
                wmove(win, 1, 2);
		if(ch == '\n') wprintw(win, ">%c<", ch);
		if(color == TRUE) wattron(win, color_pair);
		wprintw(win, "%s\n", msg);
		if(color == TRUE) wstandend(win);
                wmove(win, 3, (w / 2) - 2);
                wprintw(win, "[");
                
		if(color == TRUE) wattron(win, COLOR_PAIR(COLOR_CYAN) | A_BOLD);
                else wattron(win, A_REVERSE);
		
		wprintw(win, "OK");
                if(color == TRUE) wstandend(win);
                wprintw(win, "]\n");
                if(color == TRUE) box(win, ACS_VLINE, ACS_HLINE);
                wrefresh(win);
                ch = wgetch(win);
        }
	delwin(win);
	return 0;
}

/* Code to do a yes/no style popup */
int popup_yesno(char *msg, int def) {

        int ch, x, y, w, h;	/* a character, x/y cordinates, and width/height definitions 	*/
        int yn = def;		/* yn = yes or no, set to value of def (why did I do this?)	*/
	int val;	/* return value */
	
	WINDOW *win;
		
	x = ((LINES / 2) - 5);				/* center X cordinate	*/
	y = ((COLS / 2) - (strlen(msg) / 2) - 4);	/* center Y cordinate	*/
	w = (strlen(msg) + 4);				/* figure width		*/
	h = 5;						/* all these are 5 high	*/
	
	/* don't try to create window bigger than terminal size	*/
	if(w > COLS || (w + y) > COLS)  { 
		w = COLS;
		y = 0;
	}

	win = newwin(h, w, x, y);			/* create a new window	*/
	
        noecho();					/* turn off echo keystrokes	*/
        keypad(win, TRUE);				/* enable use of keypad	*/
        ch = 'Z';					/* set char to something benign	*/
	
	/* until user hits "ENTER" or makes a selection, loop	*/
	while(ch != 13) {
                wrefresh(win);		/* refresh window for any changes	*/
                wmove(win, 1, 2);	/* move to the 2nd char on the second line	*/

                wprintw(win, "%s", msg);	/* print the message to screen	*/
		wmove(win, 3, ((w / 2) - 5));	/* move to appropo spot to print yes/no buttons	*/
		/* make decisions on what button to highlight, and print said	*/
		if(yn == TRUE) {	/* YES selected, print it bold/color	*/
			wprintw(win, "[");
			if(color == TRUE) wattron(win, COLOR_PAIR(COLOR_CYAN));
			else wattron(win, A_REVERSE);
			wattron(win, A_BOLD);
                        wprintw(win, "YES");
                        wstandend(win);
                        wprintw(win, "]");

                } else {	/* yes not selected, don't colorize/bold it.	*/
                        wprintw(win, "[YES]");
                }
	
                if(yn == FALSE) {	/* apply above comments to NO		*/
                        wprintw(win, " [");
                        if(color == TRUE) wattron(win, COLOR_PAIR(COLOR_CYAN));
			else wattron(win, A_REVERSE);
			wattron(win, A_BOLD);
                        wprintw(win, "NO");
                        wstandend(win);
                        wprintw(win, "]\n");
                } else {
                        wprintw(win, " [NO]\n");
                }

		/* draw a box around the window for visual definition	*/
                if(color == TRUE) box(win, ACS_VLINE, ACS_HLINE);
                ch = wgetch(win);		/* wait for user input	*/
		/* 
		 * depending on what user enters, set value of yn and possibly ch.
		 * ch value is set to 13 to cause an exit of the loop in case user
		 * hits "Y" or "N" to make selection instead of [ENTER]
		 */
		switch(ch) {
                        case 'y':
                        case 'Y':
                                yn = TRUE;
                                ch = 13;
                                break;
                        case KEY_LEFT:
                        case KEY_UP:
                                yn = TRUE;
                                break;
                        case 'n':
                        case 'N':
                                yn = FALSE;
                                ch = 13;
                                break;
                        case KEY_RIGHT:
                        case KEY_DOWN:
                                yn = FALSE;
                                break;
                        default:
                                break;
                }
        }
	/* see what selection was made before exiting loop, and return value accordingly	*/
	if(yn == TRUE) {
                val = TRUE;
        } else {
                val = FALSE;
        }
	delwin(win);	/* delete the window we created	*/
	return (val);	/* return	*/
}

