/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"


/* A function to display the CMD line options to screen if user hits -h	*/
void snc_showver() {
	printf("SnortConf ver %s, copyright 2002 ManISec, Inc.\n\n", VERSION);
}

void snc_showopts() {

	snc_showver();
	printf("Options are NOT case sensitive.\n\n");
	printf("-h                   This help message\n");
	printf("-v                   Print version and exit\n");
	printf("-q                   Quick Config mode\n");
	printf("-c cfgfile           specify snortconf config file\n");
	printf("-s cfgfile           specify the snort.conf file location\n");
	printf("-r \"/path/*.rules\"   specify the path to the snort rules files\n");
	printf("   NOTE: you must give \"/path/*.rules\" in \"\"'s!!\n");
	printf("-d                   Debug Mode (output to stderr)\n");
	printf("\n\n");
}

int snc_getopts(int argc, char **argv) {
	
	long int option;
	
	/* set some sane values before we take cmdline input	*/
	cf.debug = FALSE;
	
	/* of we got a config file, read said, and set options before
	 * cmd line parsing	*/
	if(cf.configfile[0] == '\0') {
		strcpy(cf.configfile, SNC_CONFIG_FILE);
	}
	
	snc_read_snc_config();

	/* Check out the command line options				*/
	while ((option = getopt(argc, argv, "hHvVqQdDc:C:r:R:")) != -1) {

		switch(option) {
			case 'h':
			case 'H':
				snc_showopts();
				return FALSE;
				break;
			case 'd':
			case 'D':
				cf.debug = TRUE;
				break;
			case 'q':
			case 'Q':
				cf.quick = TRUE;
				break;
			case 'v':
			case 'V':
				snc_showver();
				return FALSE;
				break;		
			case 'c':
			case 'C':
				strcpy(cf.configfile, optarg);
				/* thanks to Adam Mullen for pointing out need for this	*/
				snc_read_snc_config();
				break;
			case 'r':
			case 'R':
				strcpy(cf.rulesdir, optarg);
				break;
			case 's':
			case 'S':
				strcpy(cf.snort_conf, optarg);
				break;
				
			default:
				break;
		}
	}
	
	/* if we didn't get from config file or getopt some var's, set them to
	 * hardcoded defaults	*/
	if(cf.snort_conf[0] == '\0') strcpy(cf.snort_conf, SNORT_CONFIG_FILE);
	if(cf.snort_cfgdir[0] == '\0') strcpy(cf.snort_cfgdir, SNORT_CONFIG_DIR);
	if(cf.snort_logdir[0] == '\0') strcpy(cf.snort_logdir, SNORT_LOG_DIR);	
	if(cf.rulesdir[0] == '\0') strcpy(cf.rulesdir, SNORT_RULES_FILES);

	return TRUE;
}
