/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"

/* stuff all yee ole ugly ugly help calls in separate function to make
 * menu code less ugly	*/
void menu_general_help(int sel) {

	if(sel == 0) menu_help_var("Set the network interface to use when sniffing packets\n\nIE: \"eth0\" is common on Linux, \"xl0\" on OpenBSD, etc\n");
	if(sel == 1) menu_help_var("Set the directory to write logfiles too (ie: /var/log/snort)\n");
	if(sel == 2) menu_help_var("Set the user ID to switch to at runtime.\n\nNOTE: This is the NAME of the user ID, not the numeric ID\n");
	if(sel == 3) menu_help_var("Set the group ID to switch to at runtime.\n\nNOTE: This is the NAME of the group ID, not the numeric ID\n");
	if(sel == 4) menu_help_var("Ghetto messages give extra information in the alert messages.\n\n  BASIC will give  ou the CVE/BugTRAQ/Whatever reference identifiers in the alert messages, and URL will give you the full URL to the afforementioned references in the output.");
	if(sel == 5) menu_help_var("Sets the \"quiet\" output mode.  This will disable the verbose\nmessages regarding initialization and such that can become annoying in your syslogs");
	if(sel == 6) menu_help_var("Sets the Dump Payload flag.  This will cause snort to snarf the full packet, not just the headers.\n");
	if(sel == 7) menu_help_var("Statefull inspection will set snort to only pay attention to established TCP sessions.  This is usefull for weeding out a wide variety of non-successful scans and annoying tools such as snot\n\nThis is the same as running snort with the switches -z est\n");	
	if(sel == 8) menu_help_var("Quit and return to the main menu.\n");
}


/* put the printout code for this menu in separate function for clarity
 * (code gets a bit verbose with the ON/OFF status printing)	*/
void general_print_menu(int sel) {
	
	short int i;		/* used in for() loops	*/
	short int on = FALSE;	/* is this item true?	*/	

	/* yee ole list of 'things on the menu'		*/
	char *menu_item[] = {
		"Network Interface:            ",
		"Logging directory:            ", 
		"Set UID to:                   ",
		"Set GID to:                   ",
		"Ghetto Messages:              ",
		"Quiet output (no error msgs)  ", 
		"Dump Payload (log packet data)",
		"Stateful Inpsection           ",
		"[Q]uit (return to main menu)  ",
		NULL 
	};
	
	clear(); 
	print_title_bar();
	
	/* REALLY ugly amount of code here - I should do this differently to avoid it :) 	*/
	for (i = 0 ; menu_item[i] != NULL  ; i++) {
		on = FALSE;	/* until we check otherwise, assuem item is set to OFF status	*/
		/* those "checks otherwise" mentioned above	*/
		if(i == 4 && cfg.ghetto_msg == TRUE) on = TRUE;
		if(i == 5 && cfg.quiet == TRUE) on = TRUE;
		if(i == 6 && cfg.dump_payload == TRUE) on = TRUE;
		if(i == 7 && cfg.stateful == TRUE) on = TRUE;
		/* if current menu item is NOT one of the ones we don't have ON/OFF status flags for	*/
		if(i != 0 && i != 1 && i != 2 && i != 3 && i != 8) {
		/* print said status flag so user knows current state					*/
			if(on == TRUE) {
				printw("[");
				if(color == TRUE) attr_on(COLOR_PAIR(COLOR_GREEN|A_BOLD), NULL);
				printw("ON");
				if(color == TRUE) standend();
				printw("]  ");

			} else {
				printw("[");
				if(color == TRUE) attr_on(COLOR_PAIR(COLOR_RED|A_BOLD), NULL);
				printw("OFF");
				if(color == TRUE) standend();
				printw("] ");
			}
		} else {
			/* otherwise, just print \n and leading spaces		*/
			if(i == 8) printw("\n");
			printw("      ");
		}
		/* now we're down to printing out the item depending on it's selected/unselected status	*/
		if(sel == i) sel_color_set();
		printw("%s", menu_item[i]);
		if(sel == i) standend();
		/* if this is one of the items with user data that needs to be displayed, do it	*/
		if(i == 0) printw("  %s", cfg.interface);
		if(i == 1) printw("  %s", cfg.logdir);
		if(i == 2) printw("  %s", cfg.set_uid);
		if(i == 3) printw("  %s", cfg.set_gid);
		/* special case here, since ghetto_msg is both a ON/OFF flag, and a BASIC/URL flag	*/
		if(i == 4) {
			if(cfg.ghetto_msg == TRUE && cfg.ghetto_msg_type == FALSE) printw("  BASIC");
			if(cfg.ghetto_msg == TRUE && cfg.ghetto_msg_type == TRUE)  printw("  URL");
			/* if ghetto_msg is on, and is currently selected, give user "spacebar toggle" prompt	*/
			if(sel == 4 && cfg.ghetto_msg == TRUE) printw("  (spacebar toggles)");
		}
		printw("\n");	/* \n to set up next line	*/
		
	}
}

int general_config_menu() {

	int key = 0;
	int sel = 0;
	char tmp_var[4096];

	while(key != 'q') {
		
		general_print_menu(sel);
		
		key = getch();
		key = tolower(key);
	
		switch(key) {
			case KEY_UP:
			case 'u':
			case 'k':
			case 'p':
			       if(sel > 0) --sel;
			       break;
			case KEY_DOWN:
			case 'd':
			case 'j':
			case 'n':
				/* BE SURE YOU CHANGE THIS IF YOU ADD MENU ITEMS!!!!!!!!!	*/
			       if(sel < 8) ++sel;
			       break;
			case 13:
			case ' ':
			       /* see comments in menu_netconf.c for info on the popup_getval() thing	*/
			       if(sel == 0) strcpy(cfg.interface, popup_getval("Enter the name of the network interface to use (ie: \"eth0\", or \"xl0\")"));
			       if(sel == 1) {
				       strcpy(tmp_var, popup_getval("Enter the directory where snort should write its logs (ie: /var/log/snort)"));
				       if(snc_check_dir(tmp_var) != TRUE) do_popup("ERROR: That directory does not exist!", PERR);
				       else strcpy(cfg.logdir, tmp_var);
			       }
			       if(sel == 2) strcpy(cfg.set_uid, popup_getval("Enter the name of the UNIX user to switch to at runtime:"));
			       if(sel == 3) strcpy(cfg.set_gid, popup_getval("Enter the name of the UNIX group to switch to at runtime:"));
			       if(sel == 4) {	/* ghetto_msg is unique, spacebar toggles, enter flags	*/
					if(key == 13) cfg.ghetto_msg = toggle_flag(cfg.ghetto_msg);	/* flag it	*/
					if(key == ' ') {						/* toggle type	*/
						cfg.ghetto_msg = TRUE;
						cfg.ghetto_msg_type = toggle_flag(cfg.ghetto_msg_type);
					}
				}
			       /* a few toggle flags	*/
				if(sel == 5) cfg.quiet = toggle_flag(cfg.quiet);
				if(sel == 6) cfg.dump_payload = toggle_flag(cfg.dump_payload);
				if(sel == 7) cfg.stateful = toggle_flag(cfg.stateful);
				if(sel == 8) key = 'q';
			      	break; 
			case '?':
				menu_general_help(sel);
				break;
		}
			
	}
	/* again, someday we check for errors :)	*/
	return 0;
}
	
