/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"


/* stuff ugly help text stuffs in it's own function to make other code more readable	*/
void preproc_config_menu_help(int sel) {

	if(sel == 0) menu_help_var("IP defragmentation support from Dragos Ruiu.\n\nThis will cause fragmented packet streams to be unfragmented and processed normally by Snort.\n\nThe use of this plugin is HIGHLY recommended to allow detection of attacks using such opensource tools as \"fragrouter\" and the like.\n");
	if(sel == 1) menu_help_var("stream4: stateful inspection/stream reassembly for Snort\n----------------------------------------------------------------------\nUse in concert with the -z [all|est] command line switch to defeat stick/snot against TCP rules.  Also performs full TCP stream reassembly, stateful inspection of TCP streams, etc.  Can statefully detect various portscan types, fingerprinting, ECN, etc.");
	if(sel == 2) menu_help_var("tcp stream reassembly directive\n----------------------------------------------------------------------\nActivating this option will enable the reassembly option of the stream4 module.\n");
	if(sel == 3) menu_help_var("http_decode: normalize HTTP requests\n------------------------------------\nhttp_decode normalizes HTTP requests from remote machines by converting any %%XX character substitutions to their ASCII equivalent. This is very useful for doing things like defeating hostile attackers trying to stealth themselves from IDSs by mixing these substitutions in with the request.\n");
	if(sel == 4) menu_help_var("unidecode: normalize HTTP/detect UNICODE attacks\n------------------------------------------------\nWorks much the same as http_decode, but does a better job of categorizing and identifying UNICODE attacks, recommended as a potential replacement for http_decode.");
	if(sel == 5) menu_help_var("rpc_decode: normalize RPC traffic\n---------------------------------\nRPC may be sent in alternate encodings besides the usual 4-byte encoding that is used by default.  This preprocessor normalizes RPC traffic in much the same way as the http_decode preprocessor.  This plugin takes the ports numbers that RPC services are running on as arguments.\n");
	if(sel == 6) menu_help_var("Back Orifice detector\n-------------------------\nDetects Back Orifice traffic on the network.  This preprocessor uses the Back Orifice \"encryption\" algorithm to search for traffic conforming to the Back Orifice protocol (not BO2K).\n");
	if(sel == 7) menu_help_var("telnet_decode: Telnet negotiation string normalizer\n---------------------------------------------------\nThis preprocessor \"normalizes\" telnet negotiation strings from telnet and ftp traffic.  It works in much the same way as the http_decode preprocessor, searching for traffic that breaks up the normal data stream of a protocol and replacing it with a normalized representation of that traffic so that the \"content\" pattern matching subsystem can work without requiring modifications.\n");
	if(sel == 8) menu_help_var("Spade: the Statistical Packet Anomaly Detection Engine\n-------------------------------------------------------\nREAD the README.Spade file before using this plugin!\n\nSee http://www.silicondefense.com/spice/ for more info\n\nSpade is a Snort plugin to report unusual, possibly\nsuspicious, packets. Spade will review the packets received by Snort, find those of interest (TCP SYNs into your homenets, if any), and report those packets that it believes are anomalous along with an anomaly score.\n");
	if(sel == 9) menu_help_var("arpspoof\n----------------------------------------\nExperimental ARP detection code from Jeff Nathan, detects ARP attacks, unicast ARP requests, and specific ARP mapping monitoring.  To make use of this preprocessor you must specify the IP and hardware address of hosts on the same layer 2 segment as you. Specify one host IP MAC combo per line.Also takes a \"-unicast\" option to turn on unicast ARP request detection.\n\nExamples: \n\n# preprocessor arpspoof\n# preprocessor arpspoof_detect_host: 192.168.40.1 f0:0f:00:f0:0f:00\n");
	if(sel == 11 || sel == 12) menu_help_var("portscan: detect a variety of portscans\n---------------------------------------\nportscan preprocessor by Patrick Mullen <p_mullen@linuxrc.net> This preprocessor detects UDP packets or TCP SYN packets going to four different ports in less than three seconds. \"Stealth\" TCP packets are always detected, regardless of these settings.\n\nUse portscan-ignorehosts to ignore TCP SYN and UDP \"scans\" from specific networks or hosts to reduce false alerts. It is typical to see many false alerts from DNS servers so you may want to add your DNS servers here. You can all multiple hosts/networks in a whitespace-delimited list.\n");
						
}

/* yee ole 'print stuff to screen'  code */
int preproc_print_menu(int sel) {

	short int i, on = FALSE;	

	char *menu_item[] = {
		"Defragment packets        (RECOMMENDED)  ", 
		"TCP Statefull Inspection  (RECOMMENDED)  ",
		"TCP Stream Reassembly                    ",
		"HTTP Decoder              (RECOMMENDED)  ",
		"Unicode Decoder                          ", 
		"RPC Decoder               (RECOMMENDED)  ",
		"Back Oriface Detector                    ",
		"Telent Decoder            (RECOMMENDED)  ",
		"Spade Statistical Anomoly Detector       \n",
		"ARP Spoofing Detector     (EXPERIMENTAL) ",
		"ARP Spoofing Hostlist     (EXPERIMENTAL) \n",
		"Portscan Detector                        ",
		"Portscan Detector Ignored Hostlist:      ",
		"[Q]uit (return to main menu)             ",
		NULL 
	};
	
	clear(); /* clear yee ole ncurses screen */

	print_title_bar();
	
	/* REALLY ugly amount of code here - I should do this differently to avoid it :) */
	for (i = 0 ; menu_item[i] != NULL  ; i++) {
		on = FALSE;
		if(i == 0 && plug.frag2 == TRUE) on = TRUE;
		if(i == 1 && plug.stream4 == TRUE) on = TRUE;
		if(i == 2 && plug.stream4_reass == TRUE) on = TRUE;
		if(i == 3 && plug.http_decode == TRUE) on = TRUE;
		if(i == 4 && plug.unidecode == TRUE) on = TRUE;
		if(i == 5 && plug.rpc_decode == TRUE) on = TRUE;
		if(i == 6 && plug.bo_detect == TRUE) on = TRUE;
		if(i == 7 && plug.telnet_decode == TRUE) on = TRUE;
		if(i == 8 && plug.spade == TRUE) on = TRUE;
		if(i == 9 && plug.arp_spoof == TRUE) on = TRUE;
		if(i == 10 && plug.arp_detect_host == TRUE) on = TRUE;
		if(i == 11 && plug.portscan == TRUE) on = TRUE;
		if(i == 12 && plug.portscan == TRUE) on = TRUE;
		
		if(i != 10 && i != 12 && i != 13) {
			if(on == TRUE) {
				if(color == TRUE) {
					printw("[");
					attr_on(COLOR_PAIR(COLOR_GREEN|A_BOLD), NULL);
					printw("ON");
					standend();
					printw("]  ");
				} else {
				printw("[ON]  ");
				}
			} else {
				if(color == TRUE) {
					printw("[");
					attr_on(COLOR_PAIR(COLOR_RED|A_BOLD), NULL);
					printw("OFF");
					standend();
					printw("] ");
				} else {
				printw("[OFF] ");
				}
			}
		} else {
			printw("      ");
		}
		
		if(sel == i) sel_color_set();
		printw("%s", menu_item[i]);
		if(sel == i) standend();
		if(i == 12) printw("[%s]\n", plug.portscan_ignorehosts);
		printw("\n");
		
	}
	return 0;

}


int preproc_config_menu() {
	int key = 0;
	int sel = 0; 

	while(key != 'q') {
		preproc_print_menu(sel);
		key = getch();
		key = tolower(key);
	
		switch(key) {
			case '?':
				 preproc_config_menu_help(sel);
				break;
			case KEY_UP:
			case 'u':
			case 'k':
			case 'p':
				if(sel > 0) --sel;
				break;
			case KEY_DOWN:
			case 'd':
			case 'j':
			case 'n':
				if(sel < 13) ++sel;
				break;
			case 13:
			case ' ':
				if(sel == 0) plug.frag2 = toggle_flag(plug.frag2);
				if(sel == 1) plug.stream4 = toggle_flag(plug.stream4);
				if(sel == 2) plug.stream4_reass = toggle_flag(plug.stream4_reass);
				if(sel == 3) plug.http_decode = toggle_flag(plug.http_decode);
				if(sel == 4) plug.unidecode = toggle_flag(plug.unidecode);	
				if(sel == 5) plug.rpc_decode = toggle_flag(plug.rpc_decode);	
				if(sel == 6) plug.bo_detect = toggle_flag(plug.bo_detect);	
				if(sel == 7) plug.telnet_decode = toggle_flag(plug.telnet_decode);	
				if(sel == 8) {
					menu_help_var("WARNING WARNING WARNING WARNING:\n\nSPADE is no longer part of the main Snort distribution as of Snort v1.9.0.\n\nIf you did not build your own Snort + SPADE from source tree you patched yourself, then THIS WILL NOT WORK FOR YOU!!!!\n\nYou have been warned.");
					if(plug.spade != TRUE && do_popup("Did you read the previous warning, and understand what it meant?", YNN) == TRUE) {
						plug.spade = toggle_flag(plug.spade);	
					} else {
						plug.spade = FALSE;
					}
				}

				if(sel == 9 || sel == 10) do_popup("Configuration of this item not yet supported by SnortConf", HIT_OK);
			
				/*	arp spoofing detector not available in current version (cuz I'm too lazy to code it right now) !!!!!!!!!
	 			if(sel == 9) {
					if(plug.arp_spoof != TRUE && do_popup("This is EXPERIMENTAL code!  Are you sure?", YNN) == TRUE) {
						plug.arp_spoof = toggle_flag(plug.arp_spoof);
						touchwin(stdscr);
						refresh();
						do_popup("Then make SURE you set the ARP Spoofing Hostlist below!", HIT_OK);
						++sel;
					} else {
						plug.arp_spoof = FALSE;
					}
				}
				if(sel == 10) plug.arp_detect_host = toggle_flag(plug.arp_detect_host);	 */

				
				if(sel == 11) plug.portscan = toggle_flag(plug.portscan);	

				if(sel == 12) plug.portscan = toggle_flag(plug.portscan);
					/* yee olde stuff we don't do anymore
					{
				       strcpy(plug.portscan_ignorehosts, popup_getval("Enter CIDR block (ie: 192.168.1.0/24) or $VAR (ie: $DNS_SERVERS):"));
				       if(snc_check_cidr(plug.portscan_ignorehosts) != TRUE) {
				       		if(snc_check_validvar(plug.portscan_ignorehosts) != TRUE) {
							do_popup("ERROR: Not a known $VAR or valid IP adddress!", PERR);
							plug.portscan_ignorehosts[0] = '\0';
						}
				       }
						       
				       if(plug.portscan_ignorehosts[0] != '\0' && plug.portscan == FALSE) plug.portscan = TRUE;
				}
				*/

				if(sel == 13) key = 'q';	
		}
			
	}
	return 0;
	
}	
