/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"


/* function to write out the config file based on user choices made in config process     */
/* This should really be broken up into a set of smaller functions instead of one big one */
int snc_write_config() {

	FILE *of;
	unsigned long int i;
	
	/* determine what the path we searched for rules was - *.rules at the end	*/
	char *rule_path;
	char *ptr;
	rule_path = malloc((strlen(cf.rulesdir) + 2));
	strcpy(rule_path, cf.rulesdir);
	ptr = strrchr(rule_path, '/');
	ptr[1] = '\0';
	/* done with path determination							*/
	
	/* open yee ole file	*/	
	of = fopen(cf.snort_conf, "w");
	/* if that didn't work, bail out */
	if(of == NULL) {
		return 1;
	}
	
	/* okie, we got output file, start spitting stuff into it	*/
	fprintf(of, "# This file was generated by SnortConf version %s - Copyright 2002 ManIsec, Inc.\n", VERSION);
	fprintf(of, "#\n");
	fprintf(of, "# The default snort.conf file was used for a template for this output code\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");

	/* lots of comment text from the default snort.conf	 			*/	
        fprintf(of, "#--------------------------------------------------\n");
        fprintf(of, "#   http://www.snort.org     Snort 1.9.0 Ruleset\n");
        fprintf(of, "#     Contact: snort-sigs@lists.sourceforge.net\n");
        fprintf(of, "#--------------------------------------------------\n");
        fprintf(of, "# NOTE:This ruleset only works for 1.9.0 and later\n");
        fprintf(of, "#--------------------------------------------------\n");
        fprintf(of, "# $Id: snort.conf,v 1.110.2.2 2002/10/26 21:09:55 andrewbaker Exp $\n");
        fprintf(of, "#\n");
        fprintf(of, "###################################################\n");
        fprintf(of, "# This file contains a sample snort configuration.\n");
        fprintf(of, "# You can take the following steps to create your\n");
        fprintf(of, "# own custom configuration:\n");
        fprintf(of, "#\n");
        fprintf(of, "#  1) Set the network variables for your network\n");
        fprintf(of, "#  2) Configure preprocessors\n");
        fprintf(of, "#  3) Configure output plugins\n");
        fprintf(of, "#  4) Customize your rule set\n");
        fprintf(of, "#\n");
	
/* this section is specific to snortconf; I think it's nice to set this in cfg and not cmdline	*/
	/* we'll write these config options later in the menu system  */
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "# General Configuration Options (NOTE: these are not in\n");
	fprintf(of, "# the default snort.conf; they were configured by snortconf)\n");
	if(cfg.interface[0] != '\0') fprintf(of, "config interface: %s\n", cfg.interface); 
	if(cfg.logdir[0] != '\0') fprintf(of, "config logdir: %s\n", cfg.logdir);
	if(cfg.set_uid[0] != '\0') fprintf(of, "config set_uid: %s\n", cfg.set_uid);
	if(cfg.set_gid[0] != '\0') fprintf(of, "config set_gid: %s\n", cfg.set_gid);
	  if(cfg.ghetto_msg == TRUE && cfg.ghetto_msg_type == FALSE) fprintf(of, "config ghetto_msg: basic\n");
	  if(cfg.ghetto_msg == TRUE && cfg.ghetto_msg_type == TRUE) fprintf(of, "config ghetto_msg: url\n");
	if(cfg.quiet == TRUE) 		fprintf(of, "config quiet\n");
	if(cfg.dump_payload == TRUE)	fprintf(of, "config dump_payload\n");
	if(cfg.stateful == TRUE)	fprintf(of, "config stateful\n");	
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");

	fprintf(of, "###################################################\n");
        fprintf(of, "# Step #1: Set the network variables:\n");
        fprintf(of, "#\n");
        fprintf(of, "# You must change the following variables to reflect\n");
        fprintf(of, "# your local network. The variable is currently\n");
        fprintf(of, "# setup for an RFC 1918 address space.\n");
        fprintf(of, "#\n");
        fprintf(of, "# You can specify it explicitly as:\n");
        fprintf(of, "#\n");
        fprintf(of, "# var HOME_NET 10.1.1.0/24\n");
        fprintf(of, "#\n");
        fprintf(of, "# or use global variable $<interfacename>_ADDRESS\n");
        fprintf(of, "# which will be always initialized to IP address and\n");
        fprintf(of, "# netmask of the network interface which you run\n");
        fprintf(of, "# snort at.\n");
        fprintf(of, "#\n");
        fprintf(of, "# var HOME_NET $eth0_ADDRESS\n");
        fprintf(of, "#\n");
        fprintf(of, "# You can specify lists of IP addresses for HOME_NET\n");
        fprintf(of, "# by separating the IPs with commas like this:\n");
        fprintf(of, "#\n");
        fprintf(of, "# var HOME_NET [10.1.1.0/24,192.168.1.0/24]\n");
        fprintf(of, "#\n");
        fprintf(of, "# MAKE SURE YOU DON'T PLACE ANY SPACES IN YOUR LIST!\n");
        fprintf(of, "#\n");
        fprintf(of, "# or you can specify the variable to be any IP address\n");
        fprintf(of, "# like this:\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "# Network Information Configuration (slightly tweaked by SnortConf from default)\n");
	if(var.home_net[0] == '\0') {		/* we HOPE we never get here	*/
		fprintf(of, "var HOME_NET any\n");
		fprintf(of, "# NOTE: $INTERNAL and $EXTERNAL defined for vision.rules\n");
		fprintf(of, "# They are not required for stock snort *.rules\n");
		fprintf(of, "var INTERNAL $HOME_NET\n");
		fprintf(of, "var EXTERNAL !$HOME_NET\n");
		fprintf(of, "var EXTERNAL_NET !$HOME_NET\n");

        	fprintf(of, "# Configure your server lists.  This allows snort to only look for attacks\n");
	        fprintf(of, "# to systems that have a service up.  Why look for HTTP attacks if you are\n");
	        fprintf(of, "# not running a web server?  This allows quick filtering based on IP addresses\n");
	        fprintf(of, "# These configurations MUST follow the same configuration scheme as defined\n");
        	fprintf(of, "# above for $HOME_NET.\n");

		fprintf(of, "var SMTP_SERVERS $HOME_NET\n");
		fprintf(of, "var HTTP_SERVERS $HOME_NET\n");
		fprintf(of, "var SQL_SERVERS $HOME_NET\n");
		fprintf(of, "var DNS_SERVERS $HOME_NET\n");
	} else {	/* this is what we really want		*/
		fprintf(of, "var HOME_NET %s\n", var.home_net);
		fprintf(of, "# NOTE $INTERNAL and $EXTERNAL defined for vision.rules\n");
		fprintf(of, "# They are not required for stock snort *.rules\n");
		fprintf(of, "var INTERNAL %s\n", var.home_net);
		fprintf(of, "var EXTERNAL %s\n", var.external_net);
		fprintf(of, "var EXTERNAL_NET %s\n", var.external_net);
	        
		fprintf(of, "# Configure your server lists.  This allows snort to only look for attacks\n");
	        fprintf(of, "# to systems that have a service up.  Why look for HTTP attacks if you are\n");
        	fprintf(of, "# not running a web server?  This allows quick filtering based on IP addresses\n");
	        fprintf(of, "# These configurations MUST follow the same configuration scheme as defined\n");
        	fprintf(of, "# above for $HOME_NET.\n");
		
		fprintf(of, "var SMTP_SERVERS %s\n", var.smtp_servers);
		fprintf(of, "var HTTP_SERVERS %s\n", var.http_servers);
		fprintf(of, "var SQL_SERVERS %s\n", var.sql_servers);
		fprintf(of, "var DNS_SERVERS %s\n", var.dns_servers);
		/* need to write function so user can change this!!!!
		fprintf(of, "var TELNET_SERVERS %s\n", var.telnet_servers); */
		fprintf(of, "var TELNET_SERVERS $HOME_NET\n");
	}


        fprintf(of, "# Configure your service ports.  This allows snort to look for attacks\n");
        fprintf(of, "# destined to a specific application only on the ports that application\n");
        fprintf(of, "# runs on.  For example, if you run a web server on port 8081, set your\n");
        fprintf(of, "# HTTP_PORTS variable like this:\n");
        fprintf(of, "#\n");
        fprintf(of, "# var HTTP_PORTS 8081\n");
        fprintf(of, "#\n");
        fprintf(of, "# Port lists must either be continuous [eg 80:8080], or a single port [eg 80].\n");
        fprintf(of, "# We will adding support for a real list of ports in the future.\n");
 

	if(var.shellcode_ports[0] != '\0') {
		fprintf(of, "var SHELLCODE_PORTS %s\n", var.shellcode_ports);
	} else {
		fprintf(of, "var SHELLCODE_PORTS !80\n");
	}

	if(var.http_ports[0] != '\0') {
		fprintf(of, "var HTTP_PORTS %s\n", var.http_ports);
	} else {
		fprintf(of, "var HTTP_PORTS 80\n");
	}
	
	if(var.oracle_ports[0] != '\0') {
		fprintf(of, "var ORACLE_PORTS %s\n", var.oracle_ports);
	} else {
		fprintf(of, "var ORACLE_PORTS 1521\n");
	}

	/* need to write function so user can change this		*/
	fprintf(of, "#\n");
	fprintf(of, "# AIM servers.  AOL has a habit of adding new AIM servers, so instead of\n");
	fprintf(of, "# modifying the signatures when they do, we add them to this list of\n");
	fprintf(of, "# servers.\n");
	fprintf(of, "var AIM_SERVERS [64.12.24.0/24,64.12.25.0/24,64.12.26.14/24,64.12.28.0/24,64.12.29.0/24,64.12.161.0/24,64.12.163.0/24,205.188.5.0/24,205.188.9.0/24]\n");
	
	/* new for 1.8.7 stuff	*/
	
        fprintf(of, "\n# Path to your rules files (this can be a relative path)\n");
	fprintf(of, "var RULE_PATH %s\n", rule_path); 
	
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "# Preprocessor Configuration\n");
	fprintf(of, "#\n");

        fprintf(of, "###################################################\n");
        fprintf(of, "# Step #2: Configure preprocessors\n");
        fprintf(of, "#\n");
        fprintf(of, "# General configuration for preprocessors is of\n");
        fprintf(of, "# the form\n");
        fprintf(of, "# preprocessor <name_of_processor>: <configuration_options>\n");
        fprintf(of, "\n");
        fprintf(of, "# frag2: IP defragmentation support\n");
        fprintf(of, "# -------------------------------\n");
        fprintf(of, "# This preprocessor performs IP defragmentation.  This plugin will also detect\n");
        fprintf(of, "# people launching fragmentation attacks (usually DoS) against hosts.  No\n");
        fprintf(of, "# arguments loads the default configuration of the preprocessor, which is a\n");
        fprintf(of, "# 60 second timeout and a 4MB fragment buffer.\n");
        fprintf(of, "\n");
        fprintf(of, "# The following (comma delimited) options are available for frag2\n");
        fprintf(of, "#    timeout [seconds] - sets the number of [seconds] than an unfinished\n");
        fprintf(of, "#                        fragment will be kept around waiting for completion,\n");
        fprintf(of, "#                        if this time expires the fragment will be flushed\n");
        fprintf(of, "#    memcap [bytes] - limit frag2 memory usage to [number] bytes\n");
        fprintf(of, "#                      (default:  4194304)\n");
        fprintf(of, "#\n");
        fprintf(of, "#    min_ttl [number] - minimum ttl to accept\n");
        fprintf(of, "#\n");
        fprintf(of, "#    ttl_limit [number] - difference of ttl to accept without alerting\n");
        fprintf(of, "#                         will cause false positves with router flap\n");
        fprintf(of, "#\n");
        fprintf(of, "# Frag2 uses Generator ID 113 and uses the following SIDS\n");
        fprintf(of, "# for that GID:\n");
        fprintf(of, "#  SID     Event description\n");
        fprintf(of, "# -----   -------------------\n");
        fprintf(of, "#   1       Oversized fragment (reassembled frag > 64k bytes)\n");
        fprintf(of, "#   2       Teardrop-type attack\n");
	if(plug.frag2 == TRUE) fprintf(of, "preprocessor frag2\n");

        fprintf(of, "# stream4: stateful inspection/stream reassembly for Snort\n");
        fprintf(of, "#----------------------------------------------------------------------\n");
        fprintf(of, "# Use in concert with the -z [all|est] command line switch to defeat\n");
        fprintf(of, "# stick/snot against TCP rules.  Also performs full TCP stream\n");
        fprintf(of, "# reassembly, stateful inspection of TCP streams, etc.  Can statefully\n");
        fprintf(of, "# detect various portscan types, fingerprinting, ECN, etc.\n");
        fprintf(of, "\n");
        fprintf(of, "# stateful inspection directive\n");
        fprintf(of, "# no arguments loads the defaults (timeout 30, memcap 8388608)\n");
        fprintf(of, "# options (options are comma delimited):\n");
        fprintf(of, "#   detect_scans - stream4 will detect stealth portscans and generate alerts\n");
        fprintf(of, "#                  when it sees them when this option is set\n");
        fprintf(of, "#   detect_state_problems - detect TCP state problems, this tends to be very\n");
        fprintf(of, "#                           noisy because there are a lot of crappy ip stack\n");
        fprintf(of, "#                           implementations out there\n");
        fprintf(of, "#\n");
        fprintf(of, "#   disable_evasion_alerts - turn off the possibly noisy mitigation of\n");
        fprintf(of, "#                            overlapping sequences.\n");
        fprintf(of, "#\n");
        fprintf(of, "#\n");
        fprintf(of, "#   min_ttl [number]       - set a minium ttl that snort will accept to\n");
        fprintf(of, "#                            stream reassembly\n");
        fprintf(of, "#\n");
        fprintf(of, "#   ttl_limit [number]     - differential of the initial ttl on a session versus\n");
        fprintf(of, "#                             the normal that someone may be playing games.\n");
        fprintf(of, "#                             Routing flap may cause lots of false positives.\n");
        fprintf(of, "#\n");
        fprintf(of, "#   keepstats [machine|binary] - keep session statistics, add \"machine\" to\n");
        fprintf(of, "#                         get them in a flat format for machine reading, add\n");
        fprintf(of, "#                         \"binary\" to get them in a unified binary output\n");
        fprintf(of, "#                         format\n");
        fprintf(of, "#   noinspect - turn off stateful inspection only\n");
        fprintf(of, "#   timeout [number] - set the session timeout counter to [number] seconds,\n");
        fprintf(of, "#                      default is 30 seconds\n");
        fprintf(of, "#   memcap [number] - limit stream4 memory usage to [number] bytes\n");
        fprintf(of, "#   log_flushed_streams - if an event is detected on a stream this option will\n");
        fprintf(of, "#                         cause all packets that are stored in the stream4\n");
        fprintf(of, "#                         packet buffers to be flushed to disk.  This only\n");
        fprintf(of, "#                         works when logging in pcap mode!\n");
        fprintf(of, "#\n");
        fprintf(of, "# Stream4 uses Generator ID 111 and uses the following SIDS\n");
        fprintf(of, "# for that GID:\n");
        fprintf(of, "#  SID     Event description\n");
        fprintf(of, "# -----   -------------------\n");
        fprintf(of, "#   1       Stealth activity\n");
        fprintf(of, "#   2       Evasive RST packet\n");
        fprintf(of, "#   3       Evasive TCP packet retransmission\n");
        fprintf(of, "#   4       TCP Window violation\n");
        fprintf(of, "#   5       Data on SYN packet\n");
        fprintf(of, "#   6       Stealth scan: full XMAS\n");
        fprintf(of, "#   7       Stealth scan: SYN-ACK-PSH-URG\n");
        fprintf(of, "#   8       Stealth scan: FIN scan\n");
        fprintf(of, "#   9       Stealth scan: NULL scan\n");
        fprintf(of, "#   10      Stealth scan: NMAP XMAS scan\n");
        fprintf(of, "#   11      Stealth scan: Vecna scan\n");
        fprintf(of, "#   12      Stealth scan: NMAP fingerprint scan stateful detect\n");
        fprintf(of, "#   13      Stealth scan: SYN-FIN scan\n");
        fprintf(of, "#   14      TCP forward overlap\n");
	if(plug.stream4 == TRUE) fprintf(of, "preprocessor stream4: detect_scans, disable_evasion_alerts\n");

        fprintf(of, "# tcp stream reassembly directive\n");
        fprintf(of, "# no arguments loads the default configuration\n");
        fprintf(of, "#   Only reassemble the client,\n");
        fprintf(of, "#   Only reassemble the default list of ports (See below),\n");
        fprintf(of, "#   Give alerts for \"bad\" streams\n");
        fprintf(of, "#\n");
        fprintf(of, "# Available options (comma delimited):\n");
        fprintf(of, "#   clientonly - reassemble traffic for the client side of a connection only\n");
        fprintf(of, "#   serveronly - reassemble traffic for the server side of a connection only\n");
        fprintf(of, "#   both - reassemble both sides of a session\n");
        fprintf(of, "#   noalerts - turn off alerts from the stream reassembly stage of stream4\n");
        fprintf(of, "#   ports [list] - use the space separated list of ports in [list], \"all\"\n");
        fprintf(of, "#                  will turn on reassembly for all ports, \"default\" will turn\n");
        fprintf(of, "#                  on reassembly for ports 21, 23, 25, 53, 80, 143, 110, 111\n");
        fprintf(of, "#                  and 513\n");
	if(plug.stream4_reass == TRUE) fprintf(of, "preprocessor stream4_reassemble\n");
	

	        fprintf(of, "# http_decode: normalize HTTP requests\n");
        fprintf(of, "# ------------------------------------\n");
        fprintf(of, "# http_decode normalizes HTTP requests from remote\n");
        fprintf(of, "# machines by converting any %%XX character\n");
        fprintf(of, "# substitutions to their ASCII equivalent. This is\n");
        fprintf(of, "# very useful for doing things like defeating hostile\n");
        fprintf(of, "# attackers trying to stealth themselves from IDSs by\n");
        fprintf(of, "# mixing these substitutions in with the request.\n");
        fprintf(of, "# Specify the port numbers you want it to analyze as arguments.\n");
        fprintf(of, "#\n");
        fprintf(of, "# Major code cleanups thanks to rfp\n");
        fprintf(of, "#\n");
        fprintf(of, "# unicode          - normalize unicode\n");
        fprintf(of, "# iis_alt_unicode  - %%u encoding from iis\n");
        fprintf(of, "# double_encode    - alert on possible double encodings\n");
        fprintf(of, "# iis_flip_slash   - normalize  as /\n");
        fprintf(of, "# full_whitespace  - treat t as whitespace ( for apache )\n");
        fprintf(of, "#\n");
        fprintf(of, "# for that GID:\n");
        fprintf(of, "#  SID     Event description\n");
        fprintf(of, "# -----   -------------------\n");
        fprintf(of, "#   1       UNICODE attack\n");
        fprintf(of, "#   2       NULL byte attack\n");
	if(plug.http_decode == TRUE) fprintf(of, "preprocessor http_decode: 80 unicode iis_alt_unicode double_encode iis_flip_slash full_whitespace\n");

	/* this seems to have gone byebye in 1.9.0 	*/
	/* if(plug.unidecode == TRUE) fprintf(of, "preprocessor unidecode: 80 -unicode -cginull\n"); */

        fprintf(of, "# rpc_decode: normalize RPC traffic\n");
        fprintf(of, "# ---------------------------------\n");
        fprintf(of, "# RPC may be sent in alternate encodings besides the usual\n");
        fprintf(of, "# 4-byte encoding that is used by default.  This preprocessor\n");
        fprintf(of, "# normalized RPC traffic in much the same way as the http_decode\n");
        fprintf(of, "# preprocessor.  This plugin takes the ports numbers that RPC\n");
        fprintf(of, "# services are running on as arguments.\n");
        fprintf(of, "# The RPC decode preprocessor uses generator ID 106 and does not\n");
        fprintf(of, "# generate any SIDs at this time.\n");
	if(plug.rpc_decode == TRUE) fprintf(of, "preprocessor rpc_decode: 111 32771\n");

        fprintf(of, "# bo: Back Orifice detector\n");
        fprintf(of, "# -------------------------\n");
        fprintf(of, "# Detects Back Orifice traffic on the network.  This preprocessor\n");
        fprintf(of, "# uses the Back Orifice \"encryption\" algorithm to search for\n");
        fprintf(of, "# traffic conforming to the Back Orifice protocol (not BO2K).\n");
        fprintf(of, "# This preprocessor can take two arguments.  The first is \"-nobrute\"\n");
        fprintf(of, "# which turns off the plugin's brute forcing routine (brute forces\n");
        fprintf(of, "# the key space of the protocol to find BO traffic).  The second\n");
        fprintf(of, "# argument that can be passed to the routine is a number to use\n");
        fprintf(of, "# as the default key when trying to decrypt the traffic.  The\n");
        fprintf(of, "# default value is 31337 (just like BO).  Be aware that turning on\n");
        fprintf(of, "# the brute forcing option runs the risk of impacting the overall\n");
        fprintf(of, "# performance of Snort, you've been warned...\n");
        fprintf(of, "#\n");
        fprintf(of, "# The Back Orifice detector uses Generator ID 105 and uses the\n");
        fprintf(of, "# following SIDS for that GID:\n");
        fprintf(of, "#  SID     Event description\n");
        fprintf(of, "# -----   -------------------\n");
        fprintf(of, "#   1       Back Orifice traffic detected\n");
	if(plug.bo_detect == TRUE) fprintf(of, "preprocessor bo: -nobrute 31337\n");


        fprintf(of, "# telnet_decode: Telnet negotiation string normalizer\n");
        fprintf(of, "# ---------------------------------------------------\n");
        fprintf(of, "# This preprocessor \"normalizes\" telnet negotiation strings from\n");
        fprintf(of, "# telnet and ftp traffic.  It works in much the same way as the\n");
        fprintf(of, "# http_decode preprocessor, searching for traffic that breaks up\n");
        fprintf(of, "# the normal data stream of a protocol and replacing it with\n");
        fprintf(of, "# a normalized representation of that traffic so that the \"content\"\n");
        fprintf(of, "# pattern matching keyword can work without requiring modifications.\n");
        fprintf(of, "# This preprocessor requires no arguments.\n");
        fprintf(of, "# Portscan uses Generator ID 109 and does not generate any SID currently.\n");
	if(plug.telnet_decode == TRUE) fprintf(of, "preprocessor telnet_decode\n");

        fprintf(of, "# Portscan: detect a variety of portscans\n");
        fprintf(of, "# ---------------------------------------\n");
        fprintf(of, "# portscan preprocessor by Patrick Mullen <p_mullen@linuxrc.net>\n");
        fprintf(of, "# This preprocessor detects UDP packets or TCP SYN packets going to\n");
        fprintf(of, "# four different ports in less than three seconds. \"Stealth\" TCP\n");
        fprintf(of, "# packets are always detected, regardless of these settings.\n");
        fprintf(of, "# Portscan uses Generator ID 100 and uses the following SIDS for that GID:\n");
        fprintf(of, "#  SID     Event description\n");
        fprintf(of, "# -----   -------------------\n");
        fprintf(of, "#   1       Portscan detect\n");
        fprintf(of, "#   2       Inter-scan info\n");
        fprintf(of, "#   3       Portscan End\n");
        fprintf(of, "\n");
        fprintf(of, "# preprocessor portscan: $HOME_NET 4 3 portscan.log\n");
        fprintf(of, "\n");
        fprintf(of, "# Use portscan-ignorehosts to ignore TCP SYN and UDP \"scans\" from\n");
        fprintf(of, "# specific networks or hosts to reduce false alerts. It is typical\n");
        fprintf(of, "# to see many false alerts from DNS servers so you may want to\n");
        fprintf(of, "# add your DNS servers here. You can all multiple hosts/networks\n");
        fprintf(of, "# in a whitespace-delimited list.\n");
        fprintf(of, "#\n");
        fprintf(of, "#preprocessor portscan-ignorehosts: 0.0.0.0\n");
        fprintf(of, "\n");
        fprintf(of, "# arpspoof\n");
        fprintf(of, "#----------------------------------------\n");
        fprintf(of, "# Experimental ARP detection code from Jeff Nathan, detects ARP attacks,\n");
        fprintf(of, "# unicast ARP requests, and specific ARP mapping monitoring.  To make use\n");
        fprintf(of, "# of this preprocessor you must specify the IP and hardware address of hosts on # the same layer 2 segment as you.  Specify one host IP MAC combo per line.\n");
        fprintf(of, "# Also takes a \"-unicast\" option to turn on unicast ARP request detection.\n");
        fprintf(of, "# Arpspoof uses Generator ID 112 and uses the following SIDS for that GID:\n");
        fprintf(of, "#  SID     Event description\n");
        fprintf(of, "# -----   -------------------\n");
        fprintf(of, "#   1       Unicast ARP request\n");
        fprintf(of, "#   2       Etherframe ARP mismatch (src)\n");
        fprintf(of, "#   3       Etherframe ARP mismatch (dst)\n");
        fprintf(of, "#   4       ARP cache overwrite attack\n");
        fprintf(of, "\n");
        fprintf(of, "#preprocessor arpspoof\n");
        fprintf(of, "#preprocessor arpspoof_detect_host: 192.168.40.1 f0:0f:00:f0:0f:00\n");
        fprintf(of, "\n");
        fprintf(of, "# ASN1 Decode\n");
        fprintf(of, "#-----------------------------------------\n");
        fprintf(of, "# This is an experimental preprocessor.  ASN.1 decoder and analysis plugin\n");
        fprintf(of, "# from Andrew R. Baker.  This preprocessor will detect abuses of the ASN.1\n");
        fprintf(of, "# protocol that higher level protocols (like SSL, SNMP, x.509, etc) rely on.\n");
        fprintf(of, "# The ASN.1 decoder uses Generator ID 115 and uses the following SIDs for\n");
        fprintf(of, "# that GID:\n");
        fprintf(of, "#  SID     Event description\n");
        fprintf(of, "# -----   -------------------\n");
        fprintf(of, "#   1       Indefinite length\n");
        fprintf(of, "#   2       Invalid length\n");
        fprintf(of, "#   3       Oversized item\n");
        fprintf(of, "#   4       ASN.1 specification violation\n");
        fprintf(of, "#   5       Dataum bad length\n");
        fprintf(of, "\n");
        fprintf(of, "# preprocessor asn1_decode\n");
        fprintf(of, "\n");
        fprintf(of, "# Fnord\n");
        fprintf(of, "#-----------------------------------------\n");
        fprintf(of, "# This is an experimental preprocessor.  Polymorphic shellcode analyzer and\n");
        fprintf(of, "# detector by Dragos Ruiu.  This preprocessor will watch traffic for\n");
        fprintf(of, "# polymorphic NOP-type sleds to defeat tools like ADMutate.  The Fnord detector\n");
        fprintf(of, "# uses Generator ID 114 and the following SIDs:\n");
        fprintf(of, "#  SID     Event description\n");
        fprintf(of, "# -----   -------------------\n");
        fprintf(of, "#   1       NOP-sled detected\n");
        fprintf(of, "\n");
        fprintf(of, "# preprocessor fnord\n");
        fprintf(of, "\n");

	/* this is centric to SnortConf.  Not in default snort.conf file		*/
	if(plug.spade == TRUE) {
		fprintf(of, "#\n");
		fprintf(of, "#\n");
		fprintf(of, "# SPADE - long description elsewhere in SPADE doc's.\n");
		fprintf(of, "# the fact that you have this in your file at all implies that\n");
		fprintf(of, "# you know WTF you're doing. :)\n");
		fprintf(of, "#\n");
		/* if user specified logdir, use that	*/
		if(cfg.logdir[0] != '\0') fprintf(of, "preprocessor spade: 10.5 %s/spade.rcv %s/spade-log.txt 3 50000\n", cfg.logdir, cfg.logdir);
		/* otherwise, assume /var/log/snort/  (bad thing, I know, will fix later */
		else fprintf(of, "preprocessor spade: 10.5 /var/log/snort/spade.rcv /var/log/snort/spade-log.txt 3 50000\n");
		fprintf(of, "preprocessor spade-homenet: $HOME_NET\n");
		fprintf(of, "preprocessor spade-adapt2: 0.01 15 4 24 7\n");
		fprintf(of, "preprocessor spade-threshlearn: 200 24\n");
		/* as above 	*/
		if(cfg.logdir[0] != '\0') fprintf(of, "preprocessor spade-survey:  %s/snort/spade-survey.txt 60\n", cfg.logdir);
		else fprintf(of, "preprocessor spade-survey:  %s/snort/spade-survey.txt 60\n", cfg.logdir);
		fprintf(of, "preprocessor spade-stats: entropy uncondprob condprob\n");
		fprintf(of, "#\n");
	}
	if(plug.portscan == TRUE) {
		/* big changes for 1.9.0	
		fprintf(of, "preprocessor portscan: $HOME_NET 4 3 portscan.log\n");
		fprintf(of, "preprocessor portscan-ignorehosts: %s\n", plug.portscan_ignorehosts);
		*/

	        fprintf(of, "# Conversation\n");
        	fprintf(of, "#------------------------------------------\n");
	        fprintf(of, "# This preprocessor tracks conversations for tcp, udp and icmp traffic.  It\n");
       		fprintf(of, "# is a prerequisite for running portscan2.\n");
	        fprintf(of, "#\n");
	        fprintf(of, "# allowed_ip_protcols 1 6 17\n");
	        fprintf(of, "#      list of allowed ip protcols ( defaults to any )\n");
	        fprintf(of, "#\n");
	        fprintf(of, "# timeout [num]\n");
	        fprintf(of, "#      conversation timeout ( defaults to 60 )\n");
	        fprintf(of, "#\n");
	        fprintf(of, "#\n");
	        fprintf(of, "# max_conversations [num]\n");
	        fprintf(of, "#      number of conversations to support at once (defaults to 65335)\n");
	        fprintf(of, "#\n");
	        fprintf(of, "#\n");
	        fprintf(of, "# alert_odd_protocols\n");
	        fprintf(of, "#      alert on protocols not listed in allowed_ip_protocols\n");
		fprintf(of, "preprocessor conversation: allowed_ip_protocols all, timeout 60, max_conversations 32000\n");

	        fprintf(of, "# Portscan2\n");
	       	fprintf(of, "#-------------------------------------------\n");
	        fprintf(of, "# Portscan 2, detect portscans in a new and exciting way.\n");
	        fprintf(of, "#\n");
	        fprintf(of, "# Available options:\n");
	        fprintf(of, "#       scanners_max [num]\n");
	        fprintf(of, "#       targets_max [num]\n");
	        fprintf(of, "#       target_limit [num]\n");
	        fprintf(of, "#       port_limit [num]\n");
	        fprintf(of, "#       timeout [num]\n");
	        fprintf(of, "#       log [logdir]\n");
		fprintf(of, "preprocessor portscan2: scanners_max 3200, targets_max 5000, target_limit 5, port_limit 20, timeout 60\n");
				
	}

	/* more stuff we don't let user's set, but put in the file anyway	*/
        fprintf(of, "# Experimental Perf stats\n");
        fprintf(of, "# -----------------------\n");
        fprintf(of, "# No docs. Highly subject to change.\n");
        fprintf(of, "#\n");
        fprintf(of, "# preprocessor perfmonitor: console flow events time 10\n");

		
	
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "# Output Plugin Configuration\n");
        fprintf(of, "####################################################################\n");
        fprintf(of, "# Step #3: Configure output plugins\n");
        fprintf(of, "#\n");
        fprintf(of, "# Uncomment and configure the output plugins you decide to use.\n");
        fprintf(of, "# General configuration for output plugins is of the form:\n");
        fprintf(of, "#\n");
        fprintf(of, "# output <name_of_plugin>: <configuration_options>\n");
        fprintf(of, "#\n");
	if(output.syslog == TRUE) {
		fprintf(of, "output alert_syslog: LOG_AUTH LOG_ALERT LOG_NDELAY\n");
	}

        fprintf(of, "# log_tcpdump: log packets in binary tcpdump format\n");
        fprintf(of, "# -------------------------------------------------\n");
        fprintf(of, "# The only argument is the output file name.\n");
        fprintf(of, "#\n");
	if(output.tcpdump == TRUE) {
		fprintf(of, "output log_tcpdump: snort.log\n");
	}


        fprintf(of, "# database: log to a variety of databases\n");
        fprintf(of, "# ---------------------------------------\n");
        fprintf(of, "# See the README.database file for more information about configuring\n");
        fprintf(of, "# and using this plugin.\n");
        fprintf(of, "#\n");
        fprintf(of, "# output database: log, mysql, user=root password=test dbname=db host=localhost\n");
        fprintf(of, "# output database: alert, postgresql, user=snort dbname=snort\n");
        fprintf(of, "# output database: log, unixodbc, user=snort dbname=snort\n");
        fprintf(of, "# output database: log, mssql, dbname=snort user=snort password=test\n");

	if(output.database == TRUE) {
		switch(output.db_type) {
			case MYSQL:
				fprintf(of, "output database: log, mysql, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
				break;
			case PGSQL:
				fprintf(of, "output database: log, postgresql, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
				break;
			case MSSQL:
				fprintf(of, "output database: log, mssql, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
				break;
			case ODBC:
				fprintf(of, "output database: log, odbc, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
			case ORACLE:
				fprintf(of, "output database: log, oracle, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
				break;
		}
		if(output.db_port[0] != '\0') {
			fprintf(of, " port=%s\n", output.db_port);
		} else {
			fprintf(of, "\n");
		}
	}

        fprintf(of, "# xml: xml logging\n");
        fprintf(of, "# ----------------\n");
        fprintf(of, "# See the README.xml file for more information about configuring\n");
        fprintf(of, "# and using this plugin.\n");
        fprintf(of, "#\n");
        fprintf(of, "# output xml: log, file=/var/log/snortxml\n");
	if(output.xml == TRUE) {
		fprintf(of, "output xml: log, file=%s/snortxml\n", cfg.logdir);
	}

        fprintf(of, "# unified: Snort unified binary format alerting and logging\n");
        fprintf(of, "# -------------------------------------------------------------\n");
        fprintf(of, "# The unified output plugin provides two new formats for logging\n");
        fprintf(of, "# and generating alerts from Snort, the \"unified\" format.  The\n");
        fprintf(of, "# unified format is a straight binary format for logging data\n");
        fprintf(of, "# out of Snort that is designed to be fast and efficient.  Used\n");
        fprintf(of, "# with barnyard (the new alert/log processor), most of the overhead\n");
        fprintf(of, "# for logging and alerting to various slow storage mechanisms\n");
        fprintf(of, "# such as databases or the network can now be avoided.\n");
        fprintf(of, "#\n");
        fprintf(of, "# Check out the spo_unified.h file for the data formats.\n");
        fprintf(of, "#\n");
        fprintf(of, "# Two arguments are supported.\n");
        fprintf(of, "#    filename - base filename to write to (current time_t is appended)\n");
        fprintf(of, "#    limit    - maximum size of spool file in MB (default: 128)\n");
        fprintf(of, "#\n");
        fprintf(of, "# output alert_unified: filename snort.alert, limit 128\n");
        fprintf(of, "# output log_unified: filename snort.log, limit 128\n");
	if(output.unified == TRUE) {
		fprintf(of, "output alert_unified: filename snort.alert, limit 128\n");
		fprintf(of, "output log_unified: filename snort.log, limit 128\n");
	}


        fprintf(of, "# trap_snmp: SNMP alerting for Snort\n");
        fprintf(of, "# -------------------------------------------------------------\n");
        fprintf(of, "# Read the README.SNMP file for more information on enabling and using this\n");
        fprintf(of, "# plug-in.\n");
        fprintf(of, "#\n");
        fprintf(of, "#\n");
        fprintf(of, "\n");
        fprintf(of, "#The trap_snmp plugin accepts the following notification options\n");
        fprintf(of, "# [c],[p[m|s]]\n");
        fprintf(of, "# where,\n");
        fprintf(of, "#     c : Generate compact notifications. (Saves on bandwidth by\n");
        fprintf(of, "#         not reporting MOs for which values are unknown, not\n");
        fprintf(of, "#         available or, not applicable). By default this option is reset.\n");
        fprintf(of, "#     p : Generate a print of the invariant part of the offending packet.\n");
        fprintf(of, "#         This can be used to track the packet across the Internet.\n");
        fprintf(of, "#         By default this option is reset.\n");
        fprintf(of, "#     m : Use the MD5 algorithm to generate the packet print.\n");
        fprintf(of, "#         By default this algorithm is used.\n");
        fprintf(of, "#     s : Use the SHA1 algorithm to generate the packet print.\n");
        fprintf(of, "#\n");
        fprintf(of, "# The trap_snmp plugin requires several parameters\n");
        fprintf(of, "# The parameters depend on the SNMP version that is used (specified)\n");
        fprintf(of, "# For the SNMPv2c case the parameters will be as follows\n");
        fprintf(of, "#  alert, <sensorID>, [NotificationOptions] , {trap|inform}\n");
        fprintf(of, "#         -v <SnmpVersion> [-p <portNumber>] -c <community> <hostName>\n");
        fprintf(of, "#\n");
        fprintf(of, "# For SNMPv2c traps to the standard snmpTrap port# 162 with\n");
        fprintf(of, "# MD5-digest based packetPrint generation\n");
        fprintf(of, "#\n");
        fprintf(of, "# output trap_snmp: alert, 7, cpm, trap -v 2c -c myCommunity myTrapListener\n");
        fprintf(of, "#\n");
        fprintf(of, "# For SNMPv2c informs with the 'compact' notification option to port 999\n");
        fprintf(of, "#\n");
        fprintf(of, "# output trap_snmp: alert, 7, c, inform -v 2c -p 999 -c myCommunity myTrapListener\n");
        fprintf(of, "#\n");
        fprintf(of, "#\n");
        fprintf(of, "# For SNMPv3 traps with\n");
        fprintf(of, "# security name = snortUser\n");
        fprintf(of, "# security level = authentication and privacy\n");
        fprintf(of, "# authentication parameters :\n");
        fprintf(of, "#           authentication protocol = SHA ,\n");
        fprintf(of, "#           authentication pass phrase = SnortAuthPassword\n");
        fprintf(of, "# privacy (encryption) parameters\n");
        fprintf(of, "#           privacy protocol = DES,\n");
        fprintf(of, "#           privacy pass phrase = SnortPrivPassword\n");
        fprintf(of, "#\n");
        fprintf(of, "#output trap_snmp: alert, 7, trap -v 3 -u snortUser -l authPriv -a SHA -A SnortAuthPassword -x DES -X SnortPrivPassword myTrapListener\n"
);
        fprintf(of, "#For SNMPv3 informs with authentication and encryption to myTrapListener\n");
        fprintf(of, "#on port 999\n");
        fprintf(of, "#output trap_snmp: alert, 7, inform -v 3 -p 999 -u snortUser -l authPriv -a SHA -A SnortAuthPassword -x DES -X SnortPrivPassword myTrapListener\n");
        fprintf(of, "\n");

	if(output.snmp == TRUE) {
		/* too complicated to mess with for now */
	}	


        fprintf(of, "# You can optionally define new rule types and associate one or\n");
        fprintf(of, "# more output plugins specifically to that type.\n");
        fprintf(of, "#\n");
        fprintf(of, "# This example will create a type that will log to just tcpdump.\n");
        fprintf(of, "# ruletype suspicious\n");
        fprintf(of, "# {\n");
        fprintf(of, "#   type log\n");
        fprintf(of, "#   output log_tcpdump: suspicious.log\n");
        fprintf(of, "# }\n");
        fprintf(of, "#\n");
        fprintf(of, "# EXAMPLE RULE FOR SUSPICIOUS RULETYPE:\n");
        fprintf(of, "# suspicious $HOME_NET any -> $HOME_NET 6667 (msg:\"Internal IRC Server\";)\n");
        fprintf(of, "#\n");
        fprintf(of, "# This example will create a rule type that will log to syslog\n");
        fprintf(of, "# and a mysql database.\n");
        fprintf(of, "# ruletype redalert\n");
        fprintf(of, "# {\n");
        fprintf(of, "#   type alert\n");
        fprintf(of, "#   output alert_syslog: LOG_AUTH LOG_ALERT\n");
        fprintf(of, "#   output database: log, mysql, user=snort dbname=snort host=localhost\n");
        fprintf(of, "# }\n");
        fprintf(of, "#\n");
        fprintf(of, "# EXAMPLE RULE FOR REDALERT RULETYPE\n");
        fprintf(of, "# redalert $HOME_NET any -> $EXTERNAL_NET 31337 (msg:\"Someone is being LEET\"; #   flags:A+;)\n");
        fprintf(of, "\n");
        fprintf(of, "#\n");
	fprintf(of, "# Include classification.confg and any other default includes\n");
	fprintf(of, "include %s/classification.config\n", cf.snort_cfgdir);
	fprintf(of, "include %s/reference.config\n", cf.snort_cfgdir);
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
        fprintf(of, "####################################################################\n");
        fprintf(of, "# Step #4: Customize your rule set\n");
        fprintf(of, "#\n");
        fprintf(of, "# Up to date snort rules are available at http://www.snort.org\n");
        fprintf(of, "#\n");
        fprintf(of, "# The snort web site has documentation about how to write your own\n");
        fprintf(of, "# custom snort rules.\n");
        fprintf(of, "#\n");
        fprintf(of, "# The rules included with this distribution generate alerts based on\n");
        fprintf(of, "# on suspicious activity. Depending on your network environment, your\n");
        fprintf(of, "# security policies, and what you consider to be suspicious, some of\n");
        fprintf(of, "# these rules may either generate false positives ore may be detecting\n");
        fprintf(of, "# activity you consider to be acceptable; therefore, you are\n");
        fprintf(of, "# encouraged to comment out rules that are not applicable in your\n");
        fprintf(of, "# environment.\n");
        fprintf(of, "#\n");
        fprintf(of, "# Note that using all of the rules at the same time may lead to\n");
        fprintf(of, "# serious packet loss on slower machines. YMMV, use with caution,\n");
        fprintf(of, "# standard disclaimers apply. :)\n");
        fprintf(of, "#\n");
        fprintf(of, "# The following individuals contributed many of rules in this\n");
        fprintf(of, "# distribution.\n");
        fprintf(of, "#\n");
        fprintf(of, "# Credits:\n");
        fprintf(of, "#   Ron Gula <rgula@securitywizards.com> of Network Security Wizards\n");
        fprintf(of, "#   Max Vision <vision@whitehats.com>\n");
        fprintf(of, "#   Martin Markgraf <martin@mail.du.gtn.com>\n");
        fprintf(of, "#   Fyodor Yarochkin <fygrave@tigerteam.net>\n");
        fprintf(of, "#   Nick Rogness <nick@rapidnet.com>\n");
        fprintf(of, "#   Jim Forster <jforster@rapidnet.com>\n");
        fprintf(of, "#   Scott McIntyre <scott@whoi.edu>\n");
        fprintf(of, "#   Tom Vandepoel <Tom.Vandepoel@ubizen.com>\n");
        fprintf(of, "#   Brian Caswell <bmc@snort.org>\n");
        fprintf(of, "#   Zeno <admin@cgisecurity.com>\n");
        fprintf(of, "#   Ryan Russell <ryan@securityfocus.com>\n");
        fprintf(of, "#\n");
        fprintf(of, "#=========================================\n");
        fprintf(of, "# Include all relevant rulesets here\n");
        fprintf(of, "#\n");
        fprintf(of, "# shellcode, policy, info, backdoor, and virus rulesets are\n");
        fprintf(of, "# disabled by default.  These require tuning and maintance.\n");
        fprintf(of, "# Please read the included specific file for more information.\n");
        fprintf(of, "#=========================================\n");
	fprintf(of, "# Ruleset Selections - you can comment these out as appropo\n");

	for(i = rules_files.gl_offs; i < rules_files.gl_pathc ; ++i) {
		if(rules_idx[i] == TRUE) {
			fprintf(of, "include %s\n", rules_files.gl_pathv[i]);
		} else {
			fprintf(of, "# include %s\n", rules_files.gl_pathv[i]);
		}
	}
	fclose(of);	

	return 0;
	
}


	
