/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"

/* where we stick ugly ugly code for help menu's so we don't have to look at it
 * in main menu code	*/
void network_config_menu_help(int sel) {

	if(sel == 0)  menu_help_var("Enter the value of your $HOME_NET (ie: your local network).  This must be done using CIDR notation, which is in the format of X.X.X.X/BITMASK. (Where X.X.X.X = and IP address, and /BITMASK = the number of bits in the netmask.\n\nFor reference, see the following examples:\n/24 = Class C (255.255.255.0 in your network config)\n/16 = Class B (255.255.0.0 in your network config).\n/27 1/8th of a Class C 32 hosts\n/23 2 Class C 512 hosts\n/32 Single IP address.\n\nFor example, 192.168.1.0/24 would be the 192.168.1.* class C network.  For further reference on CIDR notation please see the Snort documentation or http://www.rfc-editor.org/rfcsearch.html");
	
		
	if(sel == 1) menu_help_var("Set the value of $EXTERNAL and $EXTERNAL_NET\n\nThis value defines what range of IP addresses are EXTERNAL to your network. Typically this should be set to \"!$HOME_NET\" (ie: anything not my local network) It is recommended that you do not change it unless you know what you are doing, and why you are doing it.");
	if(sel == 2) menu_help_var("Set the value of $SMTP_SERVERS\n\nThis value can either be left as default ($HOME_NET), or defined to reflect exactly your SMTP (mail) servers in CIDR notation.  IE: you might set it to 192.168.1.5/32 if your mail server has that IP address.");
	if(sel == 3) menu_help_var("Set the value of $HTTP_SERVERS\n\nThis value can either be left as default ($HOME_NET), or defined to reflect exactly your HTTP (web) servers.  IE: you might set it to 192.168.1.55/32 if your web server has that IP address.");
	if(sel == 4) menu_help_var("Set the value of $SQL_SERVERS\n\nThis value can either be left as default ($HOME_NET) or defined to reflect exactly your SQL (database) servers.  IE: you might set it to 192.168.1.35/32 if your SQL database server has that IP address.");
	if(sel == 5) menu_help_var("Set the value of $DNS_SERVERS\n\nThis value can either be left as default ($HOME_NET) or defined to reflect exactly your DNS (name) servers.  IE: you might set it to 192.168.1.2/32 if your DNS server has that IP address.");
	if(sel == 9) menu_help_var("Quit and return to the main menu");
}

/* print out yee ole menu to user	*/
void network_config_menu_print(int sel) {
	short int i;	/* used in for loop	*/	
	
	/* yee ole list of things on the menu today	*/
	char *menu_item[] = {
		"Local Netblock(s) ($HOME_NET): ",
		"External Netblock(s):          ",
		"SMTP (mail) Servers:           ",
		"HTTP (web) Servers:            ",
		"SQL (database) Servers:        ",
		"DNS (name) Servers:            ",
		"SHELLCODE ports:               ",
		"HTTP ports:                    ",
		"ORACLE ports:                  ",
		"[Q]uit (return to main menu)   ",
		NULL
	};

	clear();		
	print_title_bar();

	/* till we run out of menu items, print them things	*/
	for(i = 0; menu_item[i] != NULL; ++i) {
		/* if we're at next-to-last menu item, add a 	*/
		/* \n so's display looks nicer			*/
		if(menu_item[(i + 1)] == NULL) printw("\n");
		printw("   ");	/* leading spaces	*/
		if(i == sel) sel_color_set();	/* if selected, rev_video	*/
		printw("%s", menu_item[i]);		/* print yee ole menu item	*/
		if(i == sel) standend();		/* un-rev video			*/
		
		/* 
		 * okie; we need to display the value of various items after the menu 
		 * selection text is printed, so we see what number in the loop we're 
		 * at and print the appropo variable so user don't get confused	
		 */
		switch(i) {
			case 0:
				printw("%s", var.home_net);
				break;
			case 1:
				printw("%s", var.external_net);
				break;
			case 2:
				printw("%s", var.smtp_servers);
				break;
			case 3:
				printw("%s", var.http_servers);
				break;
			case 4:
				printw("%s", var.sql_servers);
				break;
			case 5:
				printw("%s", var.dns_servers);
				break;
			case 6:
				printw("%s", var.shellcode_ports);
				break;
			case 7:
				printw("%s", var.http_ports);
				break;
			case 8:
				printw("%s", var.oracle_ports);
				break;
		}
		
		printw("\n");	/* add a CR to the end of the line we just constructed	*/
	}
}	


/* This is the network configuration menu				*/
int network_config_menu() {
	int key = 'Z';		/* where we store user keystrokes	*/
	short int sel = 0;	/* selected item marker			*/
	char tmp_var[4096];
	
	/* till user hits 'q', loop this fruit	*/
	while(key != 'q') {

		network_config_menu_print(sel);
		/* see comments re similar in menu_main.c	*/
		key = getch();
		key = tolower(key);

		/* check yee ole keystroke from yee ole user, and do what they ask		*/
		switch(key) {
			case KEY_DOWN:		/* see comments re similar in menu_main.c	*/
			case 'd':
			case 'j':
			case 'n':
				/* MAKE SURE YOU CHANGE THIS IF YOU ADD MENU ITEMS!!!!!!!!	*/
				if(sel < 9) ++sel;
				break;
			case KEY_UP:
			case 'u':
			case 'k':
			case 'p':
				if(sel > 0) --sel;
				break;
			case 13:
			case ' ':
				if(sel == 0) { 	/* user selected to set the HOME_NET variable (WOOHOO!)			*/
					/* take value of popup_getval(); and put it in global struct var.home_net		*/
					strcpy(var.home_net, popup_getval("Enter the CIDR block for the LOCAL network:"));
					/* set default values of the rest of the stuff for user to make their lives easier	*/
					if(snc_check_cidr(var.home_net) == FALSE) { 
						do_popup("ERROR: Invalid CIDR notation!", PERR);
						network_config_menu_help(0);
						var.home_net[0] = '\0';
					}
					
					if(var.home_net[0] != '\0') {
						
						if(var.external_net[0] == '\0') strcpy(var.external_net, "!$HOME_NET");
						if(var.smtp_servers[0] == '\0') strcpy(var.smtp_servers, "$HOME_NET");
						if(var.http_servers[0] == '\0') strcpy(var.http_servers, "$HOME_NET");
						if(var.sql_servers[0] == '\0') strcpy(var.sql_servers, "$HOME_NET");
						if(var.dns_servers[0] == '\0') strcpy(var.dns_servers, "$HOME_NET");
					}
				}
				/* the rest of these are going to be same thing; ask user for input depending on what they selected,
				 * and put it in appropo variable.  */
				if(sel == 1) {
					if(do_popup("This is normally !$HOME_NET.  Are you sure?", YNN) == TRUE) {
						strcpy(tmp_var, popup_getval("Enter the CIDR block for the EXTERNAL network"));
						if(snc_check_cidr_var(tmp_var) == FALSE) do_popup("ERROR: Invalid CIDR or unknown $VAR", PERR);
						else strcpy(var.external_net, tmp_var);
					}
				}
						
				if(sel == 2) {
					strcpy(tmp_var, popup_getval("Enter the CIDR block for local SMTP (mail) servers:"));
					if(snc_check_cidr_var(tmp_var) == FALSE) do_popup("ERROR: Invalid CIDR or unknown $VAR", PERR);
					else strcpy(var.smtp_servers, tmp_var);
				}
							
				if(sel == 3) {
					strcpy(tmp_var, popup_getval("Enter the CIDR block for local HTTP (web) servers:"));
					if(snc_check_cidr_var(tmp_var) == FALSE) do_popup("ERROR: Invalid CIDR or unknown $VAR", PERR);
					else strcpy(var.http_servers, tmp_var);
				}
				if(sel == 4) {
					strcpy(tmp_var, popup_getval("Enter the CIDR block for local SQL (database) servers:"));
					if(snc_check_cidr_var(tmp_var) == FALSE) do_popup("ERROR: Invalid CIDR or unknown $VAR", PERR);
					else strcpy(var.sql_servers, tmp_var);
				}
				if(sel == 5) {
					strcpy(tmp_var, popup_getval("Enter the CIDR block for local DNS (name) servers:"));
					if(snc_check_cidr_var(tmp_var) == FALSE) do_popup("ERROR: Invalid CIDR or unknown $VAR", PERR);
					else strcpy(var.dns_servers, tmp_var);
				}

				if(sel == 6) {
					strcpy(var.shellcode_ports, popup_getval("Enter port(s) to monitor for shellcode exploits (ie: !80):"));
				}
				if(sel == 7) {
					strcpy(var.http_ports, popup_getval("Enter port(s) to monitor for shellcode exploits (ie: 80):"));
				}
				if(sel == 8) {
					strcpy(var.oracle_ports, popup_getval("Enter port(s) which run Oracle on your network (ie: 1521):"));
				}
				
				if(sel == 9) key = 'q';	/* user asked to quit - exit yee ole while() loop	*/
				break;
			case '?':
				network_config_menu_help(sel);
			       	break;
		}
	}
	/* ah, someday.. error checking. :)	*/
	return 0;
}
